/*
 *  Copyright (C) 2001 Christophe Fergeau
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"
#include "window_notifiers.h"
#include "window.h"
#include "eel-gconf-extensions.h"
#include "prefs_strings.h"
#include "bookmarks.h"
#include "toolbar.h"
#include "embed.h"
#include "misc.h"
#include "mozilla.h"
#include "themes.h"

#include <libgnome/gnome-util.h>

extern gboolean fullscreen_active;

void main_toolbar_notifier(GConfClient *client,
			   guint cnxn_id,
			   GConfEntry *entry,
			   GaleonWindow *window);
void bookmarks_toolbars_notifier(GConfClient *client,
				 guint cnxn_id,
				 GConfEntry *entry,
				 GaleonWindow *window);
void bookmarks_menus_notifier(GConfClient *client,
			      guint cnxn_id,
			      GConfEntry *entry,
			      GaleonWindow *window);
void tabs_position_notifier(GConfClient *client,
			   guint cnxn_id,
			   GConfEntry *entry,
			   GaleonWindow *window);
void
tabs_appeareance_notifier(GConfClient *client,
			  guint cnxn_id,
			  GConfEntry *entry,
			  GaleonWindow *window);
void
statusbar_appeareance_notifier(GConfClient *client,
			       guint cnxn_id,
			       GConfEntry *entry,
			       GaleonWindow *window);
void
autobookmarks_notifier(GConfClient *client,
		       guint cnxn_id,
		       GConfEntry *entry,
		       GaleonWindow *window);

void java_bool_notifier(GConfClient *client,
			guint cnxn_id,
			GConfEntry *entry,
			GaleonWindow *window);

void proxy_notifier(GConfClient *client,
		    guint cnxn_id,
		    GConfEntry *entry,
		    GaleonWindow *window);

void generic_settings_bool_notifier(GConfClient *client,
				    guint cnxn_id,
				    GConfEntry *entry,
				    GtkWidget *widget);

void window_menubar_init_notifiers (GaleonWindow *window);

static void
window_menubar_install_notifier(GaleonWindow *window, 
				gchar *pref_name,
				GConfClientNotifyFunc notifier_cb,
				gpointer user_data);

void load_images_notifier(GConfClient *client,
			  guint cnxn_id,
			  GConfEntry *entry,
			  GaleonWindow *window);
void animate_notifier(GConfClient *client,
		      guint cnxn_id,
		      GConfEntry *entry,
		      GaleonWindow *window);
void
fullscreen_menubar_notifier (GConfClient *client,
			     guint cnxn_id,
			     GConfEntry *entry,
			     GaleonWindow *window);
void
fullscreen_toolbar_notifier (GConfClient *client,
			     guint cnxn_id,
			     GConfEntry *entry,
			     GaleonWindow *window);
void
fullscreen_bookmarks_notifier (GConfClient *client,
			       guint cnxn_id,
			       GConfEntry *entry,
			       GaleonWindow *window);
void
fullscreen_statusbar_notifier (GConfClient *client,
			       guint cnxn_id,
			       GConfEntry *entry,
			       GaleonWindow *window);

void window_title_notifier (GConfClient *client,
			    guint cnxn_id,
			    GConfEntry *entry,
			    GaleonWindow *window);

void theme_changed_notifier (GConfClient *client,
			     guint cnxn_id,
			     GConfEntry *entry,
			     GaleonWindow *window);

void stylesheet_notifier (GConfClient *client,
			  guint cnxn_id,
			  GConfEntry *entry,
			  GaleonWindow *window);

void update_ui_hack_notifier(GConfClient *client,
			     guint cnxn_id,
			     GConfEntry *entry,
			     GaleonWindow *window);
/*
 * window_menubar_install_notifier: installs the notifiers for one control 
 * in the settings menu
 */ 
static void
window_menubar_install_notifier(GaleonWindow *window, 
				gchar *pref_name,
				GConfClientNotifyFunc notifier_cb,
				gpointer user_data)
{	
	GConfClient* client = eel_gconf_client_get_global();
	GConfValue* value;

	galeon_notification_add(pref_name, 
				notifier_cb, 
				(gpointer) user_data, 
				&window->notifiers_id);

	/* set the control according to the current state */
	value = gconf_client_get (client, pref_name, NULL);
	if (value)
	{
		gconf_client_set (client, pref_name, value, NULL);
		gconf_value_free (value);
	}
}

typedef struct 
{
	char *gconf_key;
	gpointer func;
	gpointer user_data;
} SettingsNotifierInfo;

/* installs all the notifiers for the settings menu */
void
window_menubar_init_notifiers (GaleonWindow *window)
{
	SettingsNotifierInfo menubar_notifiers [] =
	{
		{CONF_FILTERING_JAVA_ENABLED, 
		 java_bool_notifier, window},
		{CONF_FILTERING_JAVASCRIPT_ENABLED, 
		 generic_settings_bool_notifier, window->enable_javascript},
		{CONF_NETWORK_PROXY_MODE,
		 proxy_notifier, window},
		{CONF_RENDERING_USE_OWN_COLORS, 
		 generic_settings_bool_notifier, window->use_own_colors},
		{CONF_RENDERING_USE_OWN_FONTS, 
		 generic_settings_bool_notifier, window->use_own_fonts},
		{CONF_FILTERING_ALLOW_POPUPS, 
		 generic_settings_bool_notifier, window->allow_popups},
		{CONF_FILTERING_ANIMATE_TYPE,
		 animate_notifier, window},
		{CONF_FILTERING_IMAGE_LOADING_TYPE,
		 load_images_notifier, window},
		{NULL, NULL, NULL}
	};
	int i;

	for (i = 0; menubar_notifiers[i].gconf_key != NULL; i++)
	{
		window_menubar_install_notifier
			(window, menubar_notifiers[i].gconf_key,
			 (GConfClientNotifyFunc)menubar_notifiers[i].func,
			 menubar_notifiers[i].user_data);
	}
}

static const struct 
{
	char *gconf_key;
	gpointer func;
}
window_notifiers [] =
{
	{CONF_TOOLBAR_STYLE, main_toolbar_notifier},
	{CONF_TOOLBAR_ICON_LIST, main_toolbar_notifier},
	{CONF_TOOLBAR_URL_LOCATION, main_toolbar_notifier},
	{CONF_TOOLBAR_SPINNER_DIR, main_toolbar_notifier},
	{CONF_TOOLBAR_THEME_DIR, theme_changed_notifier},
	{CONF_BOOKMARKS_ENTRY_WIDTH, bookmarks_toolbars_notifier},
	{CONF_BOOKMARKS_GOTO_BUTTON, bookmarks_toolbars_notifier},
	{CONF_BOOKMARKS_HISTORY, bookmarks_toolbars_notifier},
	{CONF_BOOKMARKS_CLEAR, bookmarks_toolbars_notifier},
	{CONF_BOOKMARKS_XITEMS, bookmarks_menus_notifier},
	{CONF_TABS_TABBED_POSITION, tabs_position_notifier},
	{CONF_TABS_TABBED_ALWAYS_SHOW, tabs_appeareance_notifier},
	{CONF_TABS_TABBED_SHORTEN_STYLE, tabs_appeareance_notifier},
	{CONF_TABS_TABBED_SHORTEN_CHARS, tabs_appeareance_notifier},
	{CONF_TABS_TABBED_SHORTEN_POINTS, tabs_appeareance_notifier},
	{CONF_TABS_TABBED_CLOSEBUTTON, tabs_appeareance_notifier},
	{CONF_TABS_TABBED_NEW_COLOR, tabs_appeareance_notifier},
	{CONF_TABS_TABBED_LOADING_COLOR, tabs_appeareance_notifier},
	{CONF_BOOKMARKS_AB_COUNT, autobookmarks_notifier},
	{CONF_BOOKMARKS_AB_SHORTEN, autobookmarks_notifier},
	{CONF_BOOKMARKS_AB_ENABLED, autobookmarks_notifier},
	{CONF_WINDOWS_FS_SHOW_MENUBAR, fullscreen_menubar_notifier},
	{CONF_WINDOWS_FS_SHOW_BOOKMARKS, fullscreen_bookmarks_notifier},
	{CONF_WINDOWS_FS_SHOW_TOOLBARS, fullscreen_toolbar_notifier},
	{CONF_WINDOWS_FS_SHOW_STATUSBAR, fullscreen_statusbar_notifier},
	{CONF_WINDOWS_TITLE, window_title_notifier},
	{CONF_RENDERING_USER_STYLESHEET, stylesheet_notifier},
	{"/apps/galeon/updateuihack", update_ui_hack_notifier},
	{NULL, NULL}
};

/* add gconf notifiers for the toolbars */
void
window_init_notifiers(GaleonWindow *window)
{
	int i;

	for (i = 0; window_notifiers[i].gconf_key != NULL; i++)
	{
		galeon_notification_add(
			window_notifiers[i].gconf_key,
			(GConfClientNotifyFunc)window_notifiers[i].func,
			(gpointer) window,
			&window->notifiers_id);
	}
}

void update_ui_hack_notifier(GConfClient *client,
			     guint cnxn_id,
			     GConfEntry *entry,
			     GaleonWindow *window)
{
	window_update_ui_hack (window);
}

void java_bool_notifier(GConfClient *client,
			guint cnxn_id,
			GConfEntry *entry,
			GaleonWindow *window)
{
	gtk_check_menu_item_set_active
		(GTK_CHECK_MENU_ITEM (window->enable_java),
		 gconf_value_get_bool(gconf_entry_get_value(entry)));
	gtk_widget_set_sensitive (window->java_console,
		 gconf_value_get_bool(gconf_entry_get_value(entry)));
}

void generic_settings_bool_notifier(GConfClient *client,
				    guint cnxn_id,
				    GConfEntry *entry,
				    GtkWidget *widget)
{
	gtk_check_menu_item_set_active
		(GTK_CHECK_MENU_ITEM (widget),
		 gconf_value_get_bool(gconf_entry_get_value(entry)));
}

void 
load_images_notifier(GConfClient *client,
		     guint cnxn_id,
		     GConfEntry *entry,
		     GaleonWindow *window)
{
	gint load_images = gconf_value_get_int(gconf_entry_get_value(entry));
	switch (load_images) {
	case 0:
		gtk_check_menu_item_set_active
			(GTK_CHECK_MENU_ITEM (window->load_images_always), TRUE);
		break;
	case 1:
		gtk_check_menu_item_set_active
			(GTK_CHECK_MENU_ITEM 
			 (window->load_images_from_current_server_only), TRUE);
		break;
	case 2:
		gtk_check_menu_item_set_active
			(GTK_CHECK_MENU_ITEM (window->load_images_never), TRUE);
		break;
	default:
		g_warning ("strange option for " CONF_FILTERING_IMAGE_LOADING_TYPE);
	}
}

void animate_notifier(GConfClient *client,
		     guint cnxn_id,
		     GConfEntry *entry,
		     GaleonWindow *window)
{
	gint animate = gconf_value_get_int(gconf_entry_get_value(entry));
	switch (animate)
	{
	case 0:
		gtk_check_menu_item_set_active
			(GTK_CHECK_MENU_ITEM (window->animate_always), TRUE);
		break;
	case 1:
		gtk_check_menu_item_set_active
			(GTK_CHECK_MENU_ITEM 
			 (window->animate_once_through), TRUE);
		break;
	case 2:
		gtk_check_menu_item_set_active
			(GTK_CHECK_MENU_ITEM (window->animate_never), TRUE);
		break;
	default:
		g_warning ("strange option for " CONF_FILTERING_ANIMATE_TYPE);
	}
}


/* Notifiers for the toolbars */

void main_toolbar_notifier(GConfClient *client,
			   guint cnxn_id,
			   GConfEntry *entry,
			   GaleonWindow *window)
{
	window->to_update |= MAIN_TOOLBAR;
}

void bookmarks_toolbars_notifier(GConfClient *client,
				 guint cnxn_id,
				 GConfEntry *entry,
				 GaleonWindow *window)
{
	window->to_update |= BOOKMARKS_TOOLBAR;
}

void bookmarks_menus_notifier(GConfClient *client,
			      guint cnxn_id,
			      GConfEntry *entry,
			      GaleonWindow *window)
{
	bookmarks_menu_recreate (window);
}


void
tabs_position_notifier(GConfClient *client,
		      guint cnxn_id,
		      GConfEntry *entry,
		      GaleonWindow *window)
{
	gboolean pos = gconf_value_get_bool(gconf_entry_get_value(entry));
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (window->notebook),
				  pos);
}

void
tabs_appeareance_notifier (GConfClient *client,
			   guint cnxn_id,
			   GConfEntry *entry,
			   GaleonWindow *window)
{
	window->to_update |= TABS_APPEAREANCE;
}

void
statusbar_appeareance_notifier(GConfClient *client,
			       guint cnxn_id,
			       GConfEntry *entry,
			       GaleonWindow *window)
{
	GList *w;

	for (w = all_windows; w; w = g_list_next (w))
	{
		GaleonWindow *window = (GaleonWindow *) w->data;

		window_statusbar_set_security_icon (window,
			window->active_embed->secure);
	}
}

void
autobookmarks_notifier(GConfClient *client,
		       guint cnxn_id,
		       GConfEntry *entry,
		       GaleonWindow *window)
{
	autobookmarks_generate ();
}

void
fullscreen_menubar_notifier (GConfClient *client,
			     guint cnxn_id,
			     GConfEntry *entry,
			     GaleonWindow *window)
{
	gboolean show;

	if (!fullscreen_active) return;

	show = eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_MENUBAR);
	gtk_check_menu_item_set_active 
		(GTK_CHECK_MENU_ITEM (window->view_menubar), show);
}

void
fullscreen_toolbar_notifier (GConfClient *client,
			     guint cnxn_id,
			     GConfEntry *entry,
			     GaleonWindow *window)
{
	gboolean show;

	if (!fullscreen_active) return;

	show = eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_TOOLBARS);
	gtk_check_menu_item_set_active 
		(GTK_CHECK_MENU_ITEM (window->view_toolbar), show);
}

void
fullscreen_bookmarks_notifier (GConfClient *client,
			       guint cnxn_id,
			       GConfEntry *entry,
			       GaleonWindow *window)
{
	gboolean show;

	if (!fullscreen_active) return;

	show = eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_BOOKMARKS);
	gtk_check_menu_item_set_active 
		(GTK_CHECK_MENU_ITEM (window->view_bookmarks), show);
}

void
fullscreen_statusbar_notifier (GConfClient *client,
			       guint cnxn_id,
			       GConfEntry *entry,
			       GaleonWindow *window)
{
	gboolean show;

	if (!fullscreen_active) return;

	show = eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_STATUSBAR);
	gtk_check_menu_item_set_active 
		(GTK_CHECK_MENU_ITEM (window->view_statusbar), show);
}

void
window_title_notifier (GConfClient *client,
		       guint cnxn_id,
		       GConfEntry *entry,
		       GaleonWindow *window)
{
	GList *w;

	for (w = all_windows; w != NULL; w = g_list_next (w))
	{
		GaleonWindow *window = (GaleonWindow *) (w->data);
		embed_update_page_title (window->active_embed);
	}
}

void theme_changed_notifier (GConfClient *client,
			     guint cnxn_id,
			     GConfEntry *entry,
			     GaleonWindow *window)
{
	reset_theme_pixmap_cache ();
	bookmarks_load_icons ();
	bookmarks_toolbars_notifier (client, cnxn_id, entry, window);
	bookmarks_menus_notifier (client, cnxn_id, entry, window);
	tabs_appeareance_notifier (client, cnxn_id, entry, window);
	statusbar_appeareance_notifier (client, cnxn_id, entry, window);
	main_toolbar_notifier (client, cnxn_id, entry, window);
}

void proxy_notifier(GConfClient *client,
		    guint cnxn_id,
		    GConfEntry *entry,
		    GaleonWindow *window)
{
	gint proxy = gconf_value_get_int(gconf_entry_get_value(entry));
	switch (proxy) {
	case 0:
		gtk_check_menu_item_set_active
			(GTK_CHECK_MENU_ITEM (window->no_proxy), TRUE);
		break;
	case 1:
		gtk_check_menu_item_set_active
			(GTK_CHECK_MENU_ITEM 
			 (window->manual_proxy), TRUE);
		break;
	case 2:
		gtk_check_menu_item_set_active
			(GTK_CHECK_MENU_ITEM (window->autoproxy), TRUE);
		break;
	default:
		g_warning ("strange option for " CONF_NETWORK_PROXY_MODE);
	}	
}

void stylesheet_notifier (GConfClient *client,
			  guint cnxn_id,
			  GConfEntry *entry,
			  GaleonWindow *window)
{
	gchar *usersheet = eel_gconf_get_string
				(CONF_RENDERING_USER_STYLESHEET);
	gboolean exists = TRUE;
	GList *l;

	if (!usersheet || !g_file_exists (usersheet))
		exists = FALSE;

	for (l = window->embed_list; l; l = g_list_next (l))
	{
		GaleonEmbed *embed = (GaleonEmbed *) l->data;

		if (embed->usersheet)
		{
			mozilla_remove_user_sheet (embed);
			embed->usercss_applied = FALSE;
		}
		
		if (exists)
		{
			embed->usersheet =
				mozilla_set_user_sheet (embed, usersheet);
			embed->usercss_applied = TRUE;
		}
	}

	if (usersheet) g_free (usersheet);
}
